from flask import current_app, jsonify, request
from flask_restful import Resource
from flask_restful.reqparse import RequestParser
from flask_jwt_extended import ( jwt_required, get_jwt_identity )
from application.signal_manager import signalManager
from models.app import postAppSchema,deleteAppSchema,getListAppSchema,getAppSchema,putAppSchema
from webcreator.response import ResponseCode, response_result

class AppResourceList(Resource):
    def __init__(self):
        pass
        # 特殊参数，即不是从json获取参数的接口，可以将这个注释打开
        # self.parser = RequestParser()

    def get(self):
        # 特殊参数，即不是从json获取参数的接口，可以将这个注释打开
        # self.parser.add_argument("page", type=int, location="args", default=1)
        # self.parser.add_argument("pageSize", type=int, location="args", default=15)
        # args = self.parser.parse_args()

        try:
            json_payload = request.json
            print("========>", json_payload)
            data = getListAppSchema.load(json_payload)
            result = signalManager.actionGetApp.emit(**data)
            json_dumps = getListAppSchema.dump(result)
            return jsonify(json_dumps), 200
        except Exception as e:
            current_app.logger.error(e)
            return response_result(ResponseCode.DB_ERROR)

    @jwt_required
    def post(self):
        try:
            json_payload = request.json
            data = postAppSchema.load(json_payload)
            result = signalManager.actionPostApp.emit(**data)
            json_dumps = postAppSchema.dump(result)
            return jsonify(json_dumps), 200
        except Exception as e:
            current_app.logger.error(e)
            return response_result(ResponseCode.DB_ERROR)


class AppResource(Resource):
    def __init__(self):
        pass
        # 特殊参数，即不是从json获取参数的接口，可以将这个注释打开
        # self.parser = RequestParser()

    @jwt_required
    def get(self, uuid):
        # 特殊参数，即不是从json获取参数的接口，可以将这个注释打开
        # self.parser.add_argument("page", type=int, location="args", default=1)
        # self.parser.add_argument("pageSize", type=int, location="args", default=15)
        # args = self.parser.parse_args()

        try:
            json_payload = request.json
            print("========>", uuid, json_payload)
            data = getAppSchema.load(json_payload)
            result = signalManager.actionGetApp.emit(**data)
            json_dumps = getAppSchema.dump(result)
            return jsonify(json_dumps), 200
        except Exception as e:
            current_app.logger.error(e)
            return response_result(ResponseCode.DB_ERROR)


    @jwt_required
    def put(self, uuid):
        try:
            json_payload = request.json
            print("========>", uuid, json_payload)
            data = putAppSchema.load(json_payload)
            result = signalManager.actionPutApp.emit(**data)
            json_dumps = putAppSchema.dump(result)
            return jsonify(json_dumps), 200
        except Exception as e:
            current_app.logger.error(e)
            return response_result(ResponseCode.DB_ERROR)


    @jwt_required
    def delete(self, uuid):
        try:
            json_payload = request.json
            print("========>", uuid, json_payload)
            data = deleteAppSchema.load(json_payload)
            result = signalManager.actionDeleteApp.emit(**data)
            json_dumps = deleteAppSchema.dump(result)
            return jsonify(json_dumps), 200
        except Exception as e:
            current_app.logger.error(e)
            return response_result(ResponseCode.DB_ERROR)
